// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_pixel.gen.h
 *
 * GENERATED on 2024 11 15 00:22:46 from gp_pixel.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_PIXEL_GEN_H
#define GP_PIXEL_GEN_H

/*
 * Pixel type definitions and functions.
 * Do not include directly, use gp_pixel.h
 *
 * Copyright (C) 2011      Tomas Gavenciak <gavento@ucw.cz>
 * Copyright (C) 2013-2024 Cyril Hrubis <metan@ucw.cz>
 */

/**
 * @file gp_pixel.gen.h
 * @brief A pixel defintions generated from gen/include/gfxprim_config.py
 *
 * @warning All the actual values here depends on the configuration. If you
 *          change the definitions of pixel types in the gfxprim_config.py
 *          values will be different!
 */

/**
 * @brief List of all pixel types.
 *
 * The type always contains GP_PIXEL_UNKNOWN = 0 and GP_PIXEL_MAX as the last
 * value (also the number of valid types) The types are numbered to use the
 * entire range 0 .. GP_PIXEL_MAX-1. Some pixel types have aliases defined in
 * #gp_pixel_type_alias .
 *
 * You can index the #gp_pixel_types array with a pixel type to get detailed
 * info about a pixel type at a runtime.
 *
 *
 * A pixel definition is a two layered, the top layer defines a packing i.e.
 * #gp_pixel_pack which describes pixel size and how pixels are stored in the
 * pixmap buffer, then pixel type completes the definition by adding how
 * channels are defined in each individual pixel.
 *
 * Some functions need only pixel packing to operate correctly, e.g. if you
 * need to fill in a buffer with a single pixel value, while other need to know
 * the whole pixel type, e.g. filters that operate on individual channel values.
 *
 * Note that two pixels can have the same channels, packing and size but stil
 * differ e.g. RGB888 and BGR888 or same channels, different packing and
 * size, e.g. RGB888 and xRGB8888 or same channels and size but different
 * packing e.g. G1_UB and G1_DB which differ in the direction bits are read in
 * the byte.
 *
 * Also note that on the top of this pixel channels may not be linear and hence
 * may need to be linearized before you can operate on the values. That is not
 * encoded in the pixel type but rather at the gp_pixmap level where you can
 * set a gp_gamma correction with gp_pixmap_gamma_set() independently of the
 * pixel type.
 */
typedef enum gp_pixel_type {
	/**
	 * @brief Pixel type UNKNOWN
	 *
	 * Returned when pixel matching functions such as gp_pixel_rgb_lookup()
	 * when match wasn't found.
	 */
	GP_PIXEL_UNKNOWN,
	/**
	 * @brief Pixel type RGB101010
	 *
	 * BBBBBBBBBBGGGGGGGGGGRRRRRRRRRRxx
	 */
	GP_PIXEL_RGB101010,
	/**
	 * @brief Pixel type xRGB8888
	 *
	 * BBBBBBBBGGGGGGGGRRRRRRRRxxxxxxxx
	 */
	GP_PIXEL_xRGB8888,
	/**
	 * @brief Pixel type RGBA8888
	 *
	 * AAAAAAAABBBBBBBBGGGGGGGGRRRRRRRR
	 */
	GP_PIXEL_RGBA8888,
	/**
	 * @brief Pixel type RGB888
	 *
	 * BBBBBBBBGGGGGGGGRRRRRRRR
	 */
	GP_PIXEL_RGB888,
	/**
	 * @brief Pixel type BGR888
	 *
	 * RRRRRRRRGGGGGGGGBBBBBBBB
	 */
	GP_PIXEL_BGR888,
	/**
	 * @brief Pixel type RGB555
	 *
	 * BBBBBGGGGGRRRRRx
	 */
	GP_PIXEL_RGB555,
	/**
	 * @brief Pixel type RGB565
	 *
	 * BBBBBGGGGGGRRRRR
	 */
	GP_PIXEL_RGB565,
	/**
	 * @brief Pixel type RGB666
	 *
	 * BBBBBBGGGGGGRRRRRR
	 */
	GP_PIXEL_RGB666,
	/**
	 * @brief Pixel type RGB332
	 *
	 * BBGGGRRR
	 */
	GP_PIXEL_RGB332,
	/**
	 * @brief Pixel type CMYK8888
	 *
	 * CCCCCCCCMMMMMMMMYYYYYYYYKKKKKKKK
	 */
	GP_PIXEL_CMYK8888,
	/**
	 * @brief Pixel type P2
	 *
	 * PP
	 */
	GP_PIXEL_P2,
	/**
	 * @brief Pixel type P4
	 *
	 * PPPP
	 */
	GP_PIXEL_P4,
	/**
	 * @brief Pixel type P8
	 *
	 * PPPPPPPP
	 */
	GP_PIXEL_P8,
	/**
	 * @brief Pixel type G1_DB
	 *
	 * V
	 */
	GP_PIXEL_G1_DB,
	/**
	 * @brief Pixel type G2_DB
	 *
	 * VV
	 */
	GP_PIXEL_G2_DB,
	/**
	 * @brief Pixel type G4_DB
	 *
	 * VVVV
	 */
	GP_PIXEL_G4_DB,
	/**
	 * @brief Pixel type G1_UB
	 *
	 * V
	 */
	GP_PIXEL_G1_UB,
	/**
	 * @brief Pixel type G2_UB
	 *
	 * VV
	 */
	GP_PIXEL_G2_UB,
	/**
	 * @brief Pixel type G4_UB
	 *
	 * VVVV
	 */
	GP_PIXEL_G4_UB,
	/**
	 * @brief Pixel type G8
	 *
	 * VVVVVVVV
	 */
	GP_PIXEL_G8,
	/**
	 * @brief Pixel type GA88
	 *
	 * VVVVVVVVAAAAAAAA
	 */
	GP_PIXEL_GA88,
	/**
	 * @brief Pixel type G16
	 *
	 * VVVVVVVVVVVVVVVV
	 */
	GP_PIXEL_G16,
	/** @brief Last valid pixel type has value GP_PIXEL_MAX-1 */
	GP_PIXEL_MAX,
} gp_pixel_type;

#define GP_PIXEL_UNKNOWN GP_PIXEL_UNKNOWN
#define GP_PIXEL_RGB101010 GP_PIXEL_RGB101010
#define GP_PIXEL_xRGB8888 GP_PIXEL_xRGB8888
#define GP_PIXEL_RGBA8888 GP_PIXEL_RGBA8888
#define GP_PIXEL_RGB888 GP_PIXEL_RGB888
#define GP_PIXEL_BGR888 GP_PIXEL_BGR888
#define GP_PIXEL_RGB555 GP_PIXEL_RGB555
#define GP_PIXEL_RGB565 GP_PIXEL_RGB565
#define GP_PIXEL_RGB666 GP_PIXEL_RGB666
#define GP_PIXEL_RGB332 GP_PIXEL_RGB332
#define GP_PIXEL_CMYK8888 GP_PIXEL_CMYK8888
#define GP_PIXEL_P2 GP_PIXEL_P2
#define GP_PIXEL_P4 GP_PIXEL_P4
#define GP_PIXEL_P8 GP_PIXEL_P8
#define GP_PIXEL_G1_DB GP_PIXEL_G1_DB
#define GP_PIXEL_G2_DB GP_PIXEL_G2_DB
#define GP_PIXEL_G4_DB GP_PIXEL_G4_DB
#define GP_PIXEL_G1_UB GP_PIXEL_G1_UB
#define GP_PIXEL_G2_UB GP_PIXEL_G2_UB
#define GP_PIXEL_G4_UB GP_PIXEL_G4_UB
#define GP_PIXEL_G8 GP_PIXEL_G8
#define GP_PIXEL_GA88 GP_PIXEL_GA88
#define GP_PIXEL_G16 GP_PIXEL_G16

/**
 * @brief Maximal number of channels per all defined pixel types.
 *
 * @warning The value may change if gfxprim_config.py is changed.
 */
#define GP_PIXEL_CHANS_MAX 4


/**
 * @brief A macro to get address of a pixel RGB101010 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB101010(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB101010(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel xRGB8888 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_xRGB8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_xRGB8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGBA8888 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGBA8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGBA8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGB888 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB888(pixmap, x, y) GP_PIXEL_ADDR_24BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel BGR888 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_BGR888(pixmap, x, y) GP_PIXEL_ADDR_24BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_BGR888(x) GP_PIXEL_ADDR_OFFSET_24BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGB555 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB555(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB555(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGB565 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB565(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB565(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGB666 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB666(pixmap, x, y) GP_PIXEL_ADDR_18BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB666(x) GP_PIXEL_ADDR_OFFSET_18BPP_DB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel RGB332 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_RGB332(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_RGB332(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel CMYK8888 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_CMYK8888(pixmap, x, y) GP_PIXEL_ADDR_32BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_CMYK8888(x) GP_PIXEL_ADDR_OFFSET_32BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel P2 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_P2(pixmap, x, y) GP_PIXEL_ADDR_2BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P2(x) GP_PIXEL_ADDR_OFFSET_2BPP_UB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel P4 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_P4(pixmap, x, y) GP_PIXEL_ADDR_4BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P4(x) GP_PIXEL_ADDR_OFFSET_4BPP_UB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel P8 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_P8(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_P8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G1_DB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G1_DB(pixmap, x, y) GP_PIXEL_ADDR_1BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G1_DB(x) GP_PIXEL_ADDR_OFFSET_1BPP_DB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G2_DB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G2_DB(pixmap, x, y) GP_PIXEL_ADDR_2BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G2_DB(x) GP_PIXEL_ADDR_OFFSET_2BPP_DB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G4_DB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G4_DB(pixmap, x, y) GP_PIXEL_ADDR_4BPP_DB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G4_DB(x) GP_PIXEL_ADDR_OFFSET_4BPP_DB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G1_UB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G1_UB(pixmap, x, y) GP_PIXEL_ADDR_1BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G1_UB(x) GP_PIXEL_ADDR_OFFSET_1BPP_UB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G2_UB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G2_UB(pixmap, x, y) GP_PIXEL_ADDR_2BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G2_UB(x) GP_PIXEL_ADDR_OFFSET_2BPP_UB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G4_UB at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G4_UB(pixmap, x, y) GP_PIXEL_ADDR_4BPP_UB(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G4_UB(x) GP_PIXEL_ADDR_OFFSET_4BPP_UB(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G8 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G8(pixmap, x, y) GP_PIXEL_ADDR_8BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G8(x) GP_PIXEL_ADDR_OFFSET_8BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel GA88 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_GA88(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_GA88(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros for branching on pixel_type
 */

/**
 * @brief A macro to get address of a pixel G16 at x, y in a pixmap.
 *
 * @param pixmap A pixmap.
 * @param x A x coordiate.
 * @param y A y coordiate.
 *
 * @return A pointer to the first byte of the pixel.
 */
#define GP_PIXEL_ADDR_G16(pixmap, x, y) GP_PIXEL_ADDR_16BPP(pixmap, x, y)
#define GP_PIXEL_ADDR_OFFSET_G16(x) GP_PIXEL_ADDR_OFFSET_16BPP(x)

/*
 * macros for branching on pixel_type
 */

#define GP_FN_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_RGB101010:\
			FN_NAME##_RGB101010(__VA_ARGS__);\
		break;\
		case GP_PIXEL_xRGB8888:\
			FN_NAME##_xRGB8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGBA8888:\
			FN_NAME##_RGBA8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB888:\
			FN_NAME##_RGB888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_BGR888:\
			FN_NAME##_BGR888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB555:\
			FN_NAME##_RGB555(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB565:\
			FN_NAME##_RGB565(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB666:\
			FN_NAME##_RGB666(__VA_ARGS__);\
		break;\
		case GP_PIXEL_RGB332:\
			FN_NAME##_RGB332(__VA_ARGS__);\
		break;\
		case GP_PIXEL_CMYK8888:\
			FN_NAME##_CMYK8888(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P2:\
			FN_NAME##_P2(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P4:\
			FN_NAME##_P4(__VA_ARGS__);\
		break;\
		case GP_PIXEL_P8:\
			FN_NAME##_P8(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G1_DB:\
			FN_NAME##_G1_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G2_DB:\
			FN_NAME##_G2_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G4_DB:\
			FN_NAME##_G4_DB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G1_UB:\
			FN_NAME##_G1_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G2_UB:\
			FN_NAME##_G2_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G4_UB:\
			FN_NAME##_G4_UB(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G8:\
			FN_NAME##_G8(__VA_ARGS__);\
		break;\
		case GP_PIXEL_GA88:\
			FN_NAME##_GA88(__VA_ARGS__);\
		break;\
		case GP_PIXEL_G16:\
			FN_NAME##_G16(__VA_ARGS__);\
		break;\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}

#define GP_FN_RET_PER_PIXELTYPE(FN_NAME, type, ...)\
	switch (type) { \
		case GP_PIXEL_RGB101010:\
			return FN_NAME##_RGB101010(__VA_ARGS__);\
		case GP_PIXEL_xRGB8888:\
			return FN_NAME##_xRGB8888(__VA_ARGS__);\
		case GP_PIXEL_RGBA8888:\
			return FN_NAME##_RGBA8888(__VA_ARGS__);\
		case GP_PIXEL_RGB888:\
			return FN_NAME##_RGB888(__VA_ARGS__);\
		case GP_PIXEL_BGR888:\
			return FN_NAME##_BGR888(__VA_ARGS__);\
		case GP_PIXEL_RGB555:\
			return FN_NAME##_RGB555(__VA_ARGS__);\
		case GP_PIXEL_RGB565:\
			return FN_NAME##_RGB565(__VA_ARGS__);\
		case GP_PIXEL_RGB666:\
			return FN_NAME##_RGB666(__VA_ARGS__);\
		case GP_PIXEL_RGB332:\
			return FN_NAME##_RGB332(__VA_ARGS__);\
		case GP_PIXEL_CMYK8888:\
			return FN_NAME##_CMYK8888(__VA_ARGS__);\
		case GP_PIXEL_P2:\
			return FN_NAME##_P2(__VA_ARGS__);\
		case GP_PIXEL_P4:\
			return FN_NAME##_P4(__VA_ARGS__);\
		case GP_PIXEL_P8:\
			return FN_NAME##_P8(__VA_ARGS__);\
		case GP_PIXEL_G1_DB:\
			return FN_NAME##_G1_DB(__VA_ARGS__);\
		case GP_PIXEL_G2_DB:\
			return FN_NAME##_G2_DB(__VA_ARGS__);\
		case GP_PIXEL_G4_DB:\
			return FN_NAME##_G4_DB(__VA_ARGS__);\
		case GP_PIXEL_G1_UB:\
			return FN_NAME##_G1_UB(__VA_ARGS__);\
		case GP_PIXEL_G2_UB:\
			return FN_NAME##_G2_UB(__VA_ARGS__);\
		case GP_PIXEL_G4_UB:\
			return FN_NAME##_G4_UB(__VA_ARGS__);\
		case GP_PIXEL_G8:\
			return FN_NAME##_G8(__VA_ARGS__);\
		case GP_PIXEL_GA88:\
			return FN_NAME##_GA88(__VA_ARGS__);\
		case GP_PIXEL_G16:\
			return FN_NAME##_G16(__VA_ARGS__);\
		default: GP_ABORT("Invalid PixelType %d", type);\
	}

#endif /* GP_PIXEL_GEN_H */
